use serde::{Deserialize, Serialize};
use strum::Display;

use crate::types::{
    block::BlockInfo,
    era::EraInfo,
};

#[derive(Debug, Clone, PartialEq, Eq, Display, Serialize, Deserialize)]
pub enum Action {
    Tick,
    Render,
    Resize(u16, u16),
    Suspend,
    Resume,
    Quit,
    ClearScreen,
    Error(String),
    Help,

    SetMode(crate::app::Mode),
    //MenuItem(bool),

    GetNodeName,
    GetSyncState,
    GetGenesisHash,
    GetChainName,
    GetNodeVersion,
    GetPendingExtrinsics,

    GetLatestBlock,
    GetFinalizedBlock,
    GetActiveEra,
    GetEpoch,
    GetValidators,

    SetNodeName(Option<String>),
    SetSyncState(Option<u32>, bool, bool),
    SetGenesisHash(Option<String>),
    SetChainName(Option<String>),
    SetNodeVersion(Option<String>),

    SetLatestBlock(String, BlockInfo),
    SetFinalizedBlock(String, BlockInfo),
    SetActiveEra(EraInfo),
    SetEpoch(u64, u64),
    SetValidators(Vec<String>),
    SetPendingExtrinsicsLength(usize),
}
