use color_eyre::Result;
use codec::Decode;

use crate::network::GhostRequest;
use crate::action::Action;
use crate::rpc_params;
use crate::types::{era::EraInfo, storage::GhostStorage};

#[derive(Debug)]
pub struct ActiveEraRequest<'a>(pub GhostRequest<'a>);
impl<'a> ActiveEraRequest<'a> {
    pub async fn send(self) -> Result<()> {
        let storage_key = GhostStorage::new()
            .with_module("Staking")
            .with_method("ActiveEra")
            .build_storage_key();

        let result_hex = self
            .0
            .send::<String>("state_getStorage", rpc_params![storage_key])
            .await
            .map(|r| {
                hex::decode(r.result.trim_start_matches("0x")).or::<Vec<u8>>(Ok(vec![]))
            })
            .unwrap()
            .unwrap();

        let active_era = EraInfo::decode(&mut &result_hex[..])?;
        self.0.action_tx.send(Action::SetActiveEra(active_era))?;
            
        Ok(())
    }
}
