use color_eyre::Result;
use codec::Decode;

use crate::network::GhostRequest;
use crate::action::Action;
use crate::rpc_params;
use crate::types::storage::GhostStorage;

#[derive(Debug)]
pub struct CurrentEpochRequest<'a>(pub GhostRequest<'a>);
impl<'a> CurrentEpochRequest<'a> {
    async fn do_request(&self, module: &str, method: &str) -> Result<u64> {
        let storage_key = GhostStorage::new()
            .with_module(module)
            .with_method(method)
            .build_storage_key();

        let result_hex = self
            .0
            .send::<String>("state_getStorage", rpc_params![storage_key])
            .await
            .map(|r| {
                hex::decode(r.result.trim_start_matches("0x")).or::<Vec<u8>>(Ok(vec![]))
            })
            .unwrap()
            .unwrap();
        let value = u64::decode(&mut &result_hex[..])?;
        Ok(value)
    }

    pub async fn send(self) -> Result<()> {
        let current_slot = self.do_request("Babe", "CurrentSlot").await?;
        let epoch_index = self.do_request("Babe", "EpochIndex").await?;
        let genesis_slot = self.do_request("Babe", "GenesisSlot").await?;

        let epoch_start_slot = epoch_index * 2_400 + genesis_slot;
        let progress = current_slot.saturating_sub(epoch_start_slot);
        self.0.action_tx.send(Action::SetEpoch(epoch_index, progress))?;
        Ok(())
    }
}
