use color_eyre::Result;
use serde::Deserialize;

use crate::network::GhostRequest;
use crate::action::Action;
use crate::types::params::RpcParams;

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
struct HealthCheckResponse {
    pub peers: u32,
    pub is_syncing: bool, 
    pub should_have_peers: bool
}

#[derive(Debug)]
pub struct HealthCheckRequest<'a>(pub GhostRequest<'a>);
impl<'a> HealthCheckRequest<'a> {
    pub async fn send(self) -> Result<()> {
        let (peers, is_syncing, should_have_peers) = self
            .0
            .send::<HealthCheckResponse>("system_health", RpcParams::new())
            .await
            .ok()
            .map_or((None, false, false), |response| (
                    Some(response.result.peers),
                    response.result.is_syncing,
                    response.result.should_have_peers,
            ));

        self.0.action_tx.send(Action::SetSyncState(
                peers, 
                is_syncing, 
                should_have_peers))?;
        Ok(())
    }
}
