use color_eyre::Result;
use serde::Deserialize;

use crate::network::GhostRequest;
use crate::action::Action;
use crate::rpc_params;
use crate::types::block::BlockInfo;
use crate::types::params::RpcParams;

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
struct LatestBlockResponse {
    pub block: BlockInfo, 
    // justifications: Option<String>
}

#[derive(Debug)]
pub struct LatestBlockRequest<'a>(pub GhostRequest<'a>);
impl<'a> LatestBlockRequest<'a> {
    pub async fn send(self) -> Result<()> {
        let latest_head = self
            .0
            .send::<String>("chain_getBlockHash", RpcParams::new())
            .await
            .map_or(String::new(), |response| response.result);
        let latest_block = self
            .0
            .send::<LatestBlockResponse>("chain_getBlock", rpc_params![latest_head.clone()])
            .await
            .map_or(BlockInfo::default(), |response| response.result.block);
        self.0.action_tx.send(Action::SetLatestBlock(latest_head, latest_block))?;

        Ok(())
    }
}
