use color_eyre::Result;
use codec::Decode;
use sp_core::crypto::{AccountId32, Ss58AddressFormat, Ss58Codec};

use crate::network::GhostRequest;
use crate::action::Action;
use crate::rpc_params;
use crate::types::storage::GhostStorage;

#[derive(Debug)]
pub struct ValidatorsRequest<'a>(pub GhostRequest<'a>);
impl<'a> ValidatorsRequest<'a> {
    pub async fn send(self) -> Result<()> {
        let storage_key = GhostStorage::new()
            .with_module("Session")
            .with_method("Validators")
            .build_storage_key();

        let result_hex = self.0.send::<String>("state_getStorage", rpc_params![storage_key])
            .await
            .map(|response| {
                hex::decode(response.result.trim_start_matches("0x"))
                    .ok()
                    .unwrap_or_default()
            })
            .unwrap();
        let validators = <Vec<AccountId32>>::decode(&mut &result_hex[..])
            .ok()
            .unwrap_or_default()
            .iter()
            .map(|v| v.to_ss58check_with_version(Ss58AddressFormat::custom(1996)))
            .collect::<Vec<_>>();

        self.0.action_tx.send(Action::SetValidators(validators))?;
        Ok(())
    }
}
