#[derive(Debug, Default)]
pub struct GhostStorage<'a> {
    module: &'a str,
    method: &'a str,
}

impl<'a> GhostStorage<'a> {
    pub fn new() -> Self {
        Self {
            module: "",
            method: "",
        }
    }

    pub fn with_module(mut self, module: &'a str) -> Self {
        self.module = module;
        self
    }

    pub fn with_method(mut self, method: &'a str) -> Self {
        self.method = method;
        self
    }

    pub fn build_storage_key(&self) -> String {
        let module_hex = hex::encode(sp_core::twox_128(self.module.as_bytes()));
        let method_hex = hex::encode(sp_core::twox_128(self.method.as_bytes()));

        let storage_key = format!("0x{}{}", module_hex, method_hex);
        storage_key
    }
}
